﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;

namespace WindowsFormsApplication1
{
    public class TGauss
    {
        public bool calcError;
        int maxOrder;
        double[,] m;
        double[] x;
        double[] y;

        public TGauss(int size, double[,] matrix, double[] solution)
        {
            maxOrder = size;
            m = matrix;
            y = solution;
        }

        void SwitchRows(int n)
        {
            double tempD;
            int i, j;
            for (i = n; i <= maxOrder - 2; i++)
            {
                for (j = 0; j <= maxOrder - 1; j++)
                {
                    tempD = m[i, j];
                    m[i, j] = m[i + 1, j];
                    m[i + 1, j] = tempD;
                }
                tempD = y[i];
                y[i] = y[i + 1];
                y[i + 1] = tempD;
            }
        }

        public bool Eliminate()
        {
            int i, k, l;
            calcError = false;
            for (k = 0; k <= maxOrder - 2; k++)
            {
                for (i = k; i <= maxOrder - 2; i++)
                {
                    if (Math.Abs(m[i + 1, i]) < 1e-8)
                    {
                        SwitchRows(i + 1);
                    }
                    if (m[i + 1, k] != 0.0)
                    {
                        for (l = k + 1; l <= maxOrder - 1; l++)
                        {
                            if (!calcError)
                            {
                                m[i + 1, l] = m[i + 1, l] * m[k, k] - m[k, l] * m[i + 1, k];
                                if (m[i + 1, l] > 10E260)
                                {
                                    m[i + 1, k] = 0;
                                    calcError = true;
                                }
                            }
                        }
                        y[i + 1] = y[i + 1] * m[k, k] - y[k] * m[i + 1, k];
                        m[i + 1, k] = 0;
                    }
                }
            }
            return !calcError;
        }

        public double[] Solve()
        {
            int k, l;
            double[] x = new double[maxOrder];
            for (k = maxOrder - 1; k >= 0; k--)
            {
                for (l = maxOrder - 1; l >= k; l--)
                {
                    y[k] = y[k] - x[l] * m[k, l];
                }
                if (m[k, k] != 0)
                    x[k] = y[k] / m[k, k];
                else
                    x[k] = 0;
            }
            return x;
        }
    }
}
